import sys, os, shutil, re

from const import *

# WineDoors global instances
from preferences import preferences
from packlist import packlist
from utils import Download
from utils import md5file
from utils import printNestedStuff
from wine import wine
from log import log
import tarfile

# Parser
from xml.sax import make_parser
from xml.sax.handler import ContentHandler

class ApplicationPackParse( ContentHandler ):
    """
    Parse an application pack, pack.xml file...
    this all needs to be renamed its such a bad naming scheme from the outset
    but alas we geared ourselves in that direction and we need to redo it all
    """
    def __init__( self, pack_data ):
        self.in_app_pack = False
        ContentHandler.__init__( self )
        self.pack_data = pack_data
        self.requires = {}
        self.data = []
        self.urls = []
        self.urlmd5 = None
        self.reg = False
        self.desktop = []

        self.desktop_run = None
        self.desktop_icon = None
        self.desktop_template = None

        self.pack_data['winereg'] = None
        self.pack_data['desktop'] = None
        self.pack_data['major_version'] = None
        self.pack_data['uninstall'] = None        
        self.pack_data['category'] = None
        self.pack_data['wineregpre'] = None

    def startElement( self, name, attrs ):
        if name == "ApplicationPack":
            self.in_app_pack = True
        elif name == "crack" and self.in_app_pack:
            self.crack_required = attrs.get( 'required' ) == "1"
        elif name == "requires" and self.in_app_pack:
            self.requires[attrs.get( 'name' )] = attrs.get( 'version' )
        elif name == "url" and self.in_app_pack:
            self.urlmd5 = attrs.get('md5')
        elif name == "desktop" and self.in_app_pack:
            # Take no chances with this!!
            try:
                self.desktop_template = attrs.get('template')
            except:
                pass
            try:
                self.desktop_run = attrs.get('run')
            except:
                pass
            try:
                self.desktop_icon = attrs.get('icon')
            except:
                pass

    def endElement( self, name ):
        if name == "ApplicationPack":
            self.in_app_pack = False
            self.pack_data['requires'] = self.requires
            self.pack_data['urls'] = self.urls
            if len(self.desktop) > 0:
                self.pack_data['desktop'] = self.desktop
        elif name == "crack" and self.in_app_pack:
            self.pack_data["crack_required"] = self.crack_required
            text = ''.join( self.data )
            self.pack_data[name] = text
        elif name == "url" and self.in_app_pack:
            self.urls.append( (''.join( self.data ), self.urlmd5) )
            self.urlmd5 = None
        elif name == "desktop" and self.in_app_pack: 
            if self.desktop_template:
                self.desktop.append( (self.desktop_template, self.desktop_run, self.desktop_icon) )
                self.desktop_run = None
                self.desktop_icon = None
                self.desktop_template = None
            else:
                text = ''.join( self.data )
                self.pack_data[name] = text
        elif self.in_app_pack:
            text = ''.join( self.data )
            self.pack_data[name] = text
        self.data = []

    def characters( self, string ):
        self.data.append( string.rstrip() )
            
    def error( self, exception ):
        log.Error( "%s\n" % exception )

class ApplicationPack:
    """
    This class handles the download, extraction and general organisation
    needed to install the application pack, then the wine singleton takes over
    """
    def __init__( self, operation, pack, version=None, repo=None ):
        """
        pack can be a remote file, local file or pack short name.
        pack short names require repo to be set. If no version is specified
        default version will be installed, if no default version is specified
        all versions will be installed.
        """
        self.app_pack = {}
        self.pack = []
        self.repo = repo
        self.repo_uri = None
        self.requires = []
        self.version = version
        self.operation = operation

        # We need to identify where the package is coming from, this can be 
        # a local or remote file
        if pack.startswith ( "http://" ) or pack.startswith ( "ftp://" ):
            self.pack_type = REMOTE
            self.pack_file = pack
            log.Debug("URI Pack file " + self.pack_file)
            
        elif self.repo or packlist.InAvailable( pack ):
            # This is a tough one, we need to figure out which application
            # has been selected and the appropriate version
            self.pack = packlist.GetPack(pack)
            
            if not self.version:
                try:
                    log.Debug("Setting default version: "+ self.pack['default_version'])
                    self.version = self.pack['default_version']
                except:
                    log.Debug("Installing all versions")
                    self.version = ALL
            
            if self.repo:
                self.repo_uri = packlist.GetRepoURI( self.repo )
                self.pack_type = REPO
            else:
                repo_data = packlist.GetRepo( pack )
                self.repo = ''.join( repo_data[0:1] )
                self.repo_uri = packlist.GetRepoURI( self.repo )
                self.pack_type = PACKLIST
            hit = False
            if self.version != ALL:
                for pack_version in self.pack['versions']:
                    [( majorversion, 
                       wineversion, 
                       winevariant, 
                       appdb_rating, 
                       installersize, 
                       installedsize, 
                       votes,
    	               version,
                       app_pack )] = pack_version
                    
                    if self.version in ( majorversion, version ):
                        self.pack_file = app_pack
                        log.Debug("pack file for version %s Pack file %s" % (majorversion, self.pack_file))
                        hit = True
                        break
                if not hit:
                    log.Error("Couldn't get pack file")
                    self.status = FILE_NOT_FOUND
                    return
        elif os.path.isfile( pack ):
            self.pack_type = LOCAL
            self.pack_file = pack
        else:
            log.Error( "Cannot determine the application pack type for " + pack )
            return
        
        # Run the operation
        if self.operation == INSTALL or operation == DRYRUN or operation == UPDATE:
            log.Debug( "Installing application " )
            self.InstallApplication()
        elif self.operation == UNINSTALL:
            self.repo = "installed"
            self.UninstallApplication()
        elif self.operation == DESKTOP:
            log.Debug( "Installing desktop entry " )
            self.InstallDesktopEntry()
        
    def Parse( self ):
        """
        parse the pack.xml file
        """
        self.app_pack_xml = self.pack_path + "pack.xml"
        log.Debug("Parsing file: " + self.app_pack_xml )
        handler = ApplicationPackParse( self.app_pack )
        parser = make_parser()
        parser.setContentHandler( handler )
        try:
            parser.parse( open( self.app_pack_xml ) )
        except:
            parser.parse( self.app_pack_xml )

    def Download ( self, pack_file, file_type=APP_PACK ):
        """
        We need to work out what we're downloading and to where
        """
        log.Debug("Downloading file: " + pack_file)
        
        if pack_file.startswith("http://") or pack_file.startswith("ftp://") or \
           pack_file.startswith("/") or pack_file.startswith("file://") or not self.repo:
            if pack_file.find("|") != -1:
                ren = re.compile("(.*)\|(.*?)\Z")
                m = ren.match(pack_file)
                remote_file = m.group(1)
                file_name = m.group(2)
            else:
                remote_file = pack_file
                ( directory, file_name ) = os.path.split( remote_file )
        else:
            log.Debug("Repo: "+ self.repo)
            if not self.repo_uri:
                self.repo_uri = packlist.GetRepoURI( self.repo )
            log.Debug("Repo URI: "+ self.repo_uri)
            remote_file = self.repo_uri + pack_file
            file_name = pack_file
        
        self.cache_path = os.environ['HOME'] + "/.wine-doors/apppacks/"
        
        if not os.path.isdir(self.cache_path):
            os.makedirs( self.cache_path )
        
        if file_type == APP_PACK:
            # The application contents, these must contain FREE stuff we don't
            # want lawsuits!
            if file_name.endswith(".tar.gz"):
                self.pack_folder = file_name.replace( ".tar.gz", "" )
            elif file_name.endswith(".tar.bz2"):
                self.pack_folder = file_name.replace( ".tar.bz2", "" )
            else:
                ( self.pack_folder, ext ) = os.path.splitext( file_name )

            self.pack_path = self.cache_path + self.pack_folder + "/"
            self.cache_file = self.cache_path + file_name
        elif file_type == RESOURCE:
            # Anything that an application pack instructs instructs us to download
            self.cache_file = self.pack_path + "resources/" + file_name
            if not os.path.isdir( self.pack_path + "resources" ):
                os.makedirs( self.pack_path + "resources" )
        
        # This is a hack to set up the notification of the download progress.
        # Its probably not the best way to do it
        if not wine.firstrun:
            notify_type = DL_TILESTATUS
            notify_data = (0,)
        else:
            notify_type = None
            notify_data = None

        log.Debug( "Remote file: " + remote_file )
        log.Debug( "Pack path: " + self.pack_path )
        log.Debug( "Cache file: " + self.cache_file )
        
        # Invoke the download helper
        dl = Download ( remote_file, self.cache_file, True, notify_type, notify_data )
        self.cache_file = self.cache_file.replace( ".gz", "" )
        return dl.status

    def Extract ( self ):
        cwd = os.getcwd()
        os.chdir(self.cache_path)
        log.Debug("Extracting tarball:" + self.cache_file)
        try:
            tar = tarfile.open(self.cache_file, "r")
	    files = tar.getmembers()
	    for item in files:
        	tar.extract(item)
        except:
            log.Error( "Unable to extract \"" + self.pack["shortname"] + "\"" )
        os.chdir(cwd)
                
    def InstallApplication( self ):
        """
        Install an application using an application pack
        """
        if self.version == ALL:
            for pack_version in self.pack['versions']:
                [( majorversion, 
                   wineversion, 
                   winevariant, 
                   appdb_rating, 
                   installersize, 
                   installedsize, 
                   votes,
                   version,
                   app_pack )] = pack_version
                log.Debug("Forking install for version: " + majorversion )
                self.pack_file = app_pack
                self.version = majorversion
                self.InstallApplication()
        else:
            if self.operation == DRYRUN:
                log.Debug("Doing dry run install")
            
            log.Debug("* Retrieving package to cache: " + self.pack_file)
            
            # Run our handy little tools on our pack file
            if self.Download( self.pack_file, APP_PACK ) == DOWNLOAD_FAILED:
                return
            self.Extract()
            self.Parse()
            
            # Download each URL in turn
            for item in self.app_pack['urls']:
                (url, md5) = item
                log.Debug("* Retrieving installation resource to cache: " + url)
                if self.Download( url, RESOURCE ) == DOWNLOAD_FAILED:
                    self.status = FILE_NOT_FOUND
                    return
                if md5:
                    if md5file(self.cache_file) != md5:
                        log.Error("MD5 sum doesn't match, wine-doors cache may be stale")
                else:
                    log.Warn("MD5 sum not found")
            try:
                if re.search("D:\\\\", self.app_pack['install']):
                    # Its most likely a CD-ROM
                    installer = self.app_pack['install']
                    cd = True
                elif re.search("CDROM:/", self.app_pack['install']):
                    # Erm what shall we do here?
                    pass
                    cd = True
                else:
                    installer = self.pack_path + self.app_pack['install']
                    cd = False
            except:
                installer = self.cache_file

            exe = re.compile("(.*\.(exe|ahk|sh|py|msi))(.*)", re.IGNORECASE)
            exem = exe.match(installer)

            if exem:
                installer = exem.group(1)
                installer_args = exem.group(3)
            else:
                installer_args = ""

            # Run it for real this time            
            if self.operation != DRYRUN:
                cwd = os.getcwd()
                os.chdir( self.pack_path )
                new_cwd = os.getcwd()

                # A horrible hack to import the UI stuff!!!!
                if wine.gui:
                    try:
                       ui.benign()
                    except:
                        import ui, gobject, gtk

                if self.app_pack['wineregpre'] and self.app_pack['wineregre'] != "":
                    try:
                        ui.winedoors.tree.set_cell_status((0,), "Installing registry fixes" )
                    except:
                        pass
                    self.InstallFile( REGISTRY, self.pack_path + self.app_pack['wineregpre'] )

                try:
                    ui.winedoors.tree.set_cell_status((0,), "Executing Installer" )
                except:
                    pass

                # Execute installer
                log.Debug("* About to execute: "+installer+" "+installer_args )
                log.Debug("** From folder: " + new_cwd )
                self.status = wine.Execute( installer, installer_args )
                # This stuff is really UI code and shouldn't really be here!
                if self.status == FILE_NOT_FOUND:
                    # Present the user with a gtkfilechooser if the installer file can't be found
                    if wine.gui:
                        dialog = gtk.FileChooserDialog("Locate Installer for %s %s" % (self.app_pack['name'], self.app_pack['major_version']),
                                                       None,
                                                       gtk.FILE_CHOOSER_ACTION_OPEN,
                                                       (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                                        gtk.STOCK_OPEN, gtk.RESPONSE_OK))
                        dialog.set_default_response(gtk.RESPONSE_OK)

                        filter = gtk.FileFilter()
                        filter.set_name("All installers")
                        filter.add_pattern("*.exe")
                        filter.add_pattern("*.sh")
                        filter.add_pattern("*.ahk")
                        filter.add_pattern("*.msi")
                        dialog.add_filter(filter)

                        response = dialog.run()
                        if response == gtk.RESPONSE_OK:
                            installer = dialog.get_filename()
                            dialog.destroy()
                            while gtk.events_pending():
                                gtk.main_iteration()
                            self.status = wine.Execute( installer, installer_args )
                        elif response == gtk.RESPONSE_CANCEL:
                            self.status = -1
                            log.Error("Can't find file: " + installer)
                            log.Error("User cancelled manual file location")
                        
                    else:
                        log.Error("Can't find file: " + installer)
                os.chdir(cwd)
                if wine.WaitForExit() == False:
                    wine.Kill()

            # Installer is now finished, we still need to process the registry files
            if self.app_pack['winereg'] and self.app_pack['winereg'] != "":
                if wine.gui:
                    try:
                        ui.winedoors.tree.set_cell_status((0,), "Installing registry fixes" )
                    except:
                        pass
                self.InstallFile( REGISTRY, self.pack_path + self.app_pack['winereg'] )

            # install the desktop menus
            self.InstallDesktopEntry()
            
            # FIXME: Is this still required? we need to lock down the XML 0.2-love 
            # This is here just in case the XML hasn't been updated to include
            # the major-version tag
            if not self.app_pack['major_version']:
                majorversion = self.app_pack['version']
            else:
                majorversion = self.app_pack['major_version']
            
            if wine.gui:
                try:
                    ui.winedoors.tree.set_cell_status((0,), "Updating installed status" )
                except:
                    pass
            
            if not self.status: # FIXME Installers should always return 0
                log.Debug("application.py: Adding installed file %s version %s" % (self.app_pack['shortname'], majorversion))
                packlist.AddInstalled(self.app_pack['shortname'], majorversion)
            else:
                log.Debug("application.py: Status returned: %d not adding installed file" % self.status);
            return False

    def InstallDesktopEntry( self ):
        # If pack has not already been downloaded and parsed
        # In other words, add stuff usually added by self.InstallApplication
        if self.app_pack == {}:
            if self.Download( self.pack_file, APP_PACK ) == DOWNLOAD_FAILED:
                return
            self.Extract()
            self.Parse()
        # If pack is ready to go
        if self.app_pack['desktop']:
            if wine.gui:
                try:
                    ui.winedoors.tree.set_cell_status((0,), "Installing desktop menus" )
                except:
                    pass
            if isinstance(self.app_pack['desktop'], list):
                for launcher in self.app_pack['desktop']:
                    (tpl, run, icon) = launcher
                    self.InstallFile(  LAUNCHER, 
                                       self.pack_path + tpl, 
                                       run, icon, 
                                       self.app_pack['category'] )
            else:                
                self.InstallFile(  LAUNCHER, 
                                   self.pack_path + self.app_pack['desktop'], 
                                   self.app_pack['run'], self.app_pack['icon'], 
                                   self.app_pack['category'] )

    def UninstallDesktopEntry( self ):
        log.Log( "Uninstalling desktop entry for: %s" % self.app_pack['shortname'] )
        desktop_path = os.path.expanduser( "~/.local/share/applications/" )
        desktop_file = desktop_path + self.app_pack['desktop'].split("/")[-1]
        if os.path.isfile( desktop_file ):
            os.remove( desktop_file )

    def InstallFile( self, resource_type, resource_file, run=None, icon=None, category=None ):
        desktop_path = os.path.expanduser( "~/.local/share/applications/" )
        if not os.path.isdir( desktop_path ):
            os.makedirs( desktop_path )
        if not os.path.isdir( os.environ['HOME']+"/bin/" ):
            os.makedirs( os.environ['HOME']+"/bin/" )
        # This is a bunch of hacks to get all of your installer files created nicely
        # there are so many crazy stunts in here I won't comment on them all
        if resource_type == LAUNCHER:
            if not run:
                log.Debug("No run command specified, cannot install file")
                return

            if not icon:
                log.Warn("No icon for the desktop file")

            # Santitise the inputted run file, this is a fairly useful pattern,
            # a function could be built from it
            if re.search(":\\\\", run ):
                quote_exe = re.compile("(.*\.exe) (.*)", re.IGNORECASE)
                m = quote_exe.match( run )
                if m:
                  exe = m.group(1)
                  args = " "+m.group(2)
                else:
                  exe = run
                  args = ""
                run = "env WINEDEBUG=-all WINEPREFIX="+preferences['wineroot']+" wine \""+exe+"\""+args
            elif re.search("^~", run ):
                # anytime we see a ~/... it has been installed via another process
                run = os.path.expanduser( run )
            elif os.path.isfile( run ):
                # assume run path is in ~/bin 
                ( directory, runfile ) = os.path.split( run )
                shutil.copy(self.pack_path + run, os.path.expanduser( "~/bin/"))
                run = os.path.expanduser( "~/bin/" + runfile )
                os.system("chmod +x "+run )

            desktop = open( resource_file , 'a' )
            desktop.write( 'Exec=' + run+"\n" )
                        
            # FIXME A long day in front of the computer updating all of
            # the application packs to fdo categories joy!
            if category:            
                desktop.write( 'Category=' + self.app_pack['category']+"\n" )
            if icon:
                ( directory, icon_file ) = os.path.split( icon )
                desktop.write( 'Icon=' + desktop_path + icon_file+"\n" )
                self.InstallFile( ICON, self.pack_path + icon )
            desktop.close()
            shutil.copy( resource_file, desktop_path )
        elif resource_type == ICON:
            shutil.copy( resource_file, desktop_path )
        elif resource_type == SCRIPT:
            shutil.copy( resource_file, os.environ['HOME']+"/bin/" )
        elif resource_type == REGISTRY:
            wine.Execute( "regedit", resource_file )
    
    def UninstallApplication(self):
        """
        This hasn't been completely tested to destruction yet
        """
        log.Debug("Uninstalling application: " + str(self.pack) + " " + str(self.version))
        if self.Download( self.pack_file, APP_PACK ) == DOWNLOAD_FAILED:
            return
        self.Extract()
        self.Parse()
        
        allowed = [ "msiexec" ]

        reg = re.compile("^HK")
        if not self.app_pack['uninstall']:
            uninstaller = ""
        elif reg.match(self.app_pack['uninstall']):
            uninstaller = wine.getRegistry(self.app_pack['uninstall'])
        elif self.app_pack['uninstall'].lower().startswith("c:"):
            uninstaller = self.app_pack['uninstall']
        elif self.app_pack['uninstall'].lower().split(" ")[0] in allowed:
            uninstaller = self.app_pack['uninstall']
        else:
            uninstaller = self.pack_path + self.app_pack['uninstall']
            
        exe = re.compile("(.*\.(exe|ahk|sh|py))(.*)", re.IGNORECASE)
        exem = exe.match(uninstaller)

        if exem:
            uninstaller = exem.group(1)
            args = exem.group(3)
        else:
            args = None
            
        if os.path.isfile(uninstaller) or \
           os.path.isfile( wine.ConvertPath(uninstaller) ) or \
           uninstaller.lower().split(" ")[0] in allowed :
            self.status = wine.Execute(uninstaller, args)
        else:
            log.Error("Cannot locate uninstaller: " + uninstaller)
            self.status = -1
        
        if not self.app_pack['major_version']:
            majorversion = self.app_pack['version']
        else:
            majorversion = self.app_pack['major_version']
        
        # Remove desktop entry, if there is one.
        if self.app_pack['desktop']:
            self.UninstallDesktopEntry()

        packlist.DelInstalled(self.app_pack['shortname'], majorversion)

    def Save( self, tofile = "pack.xml" ):
        """ 
        Save to pack.xml 
        """
        log.Debug("Saving application pack: " + tofile )
        f = open( tofile, "w" )
        backup = sys.stdout
        sys.stdout = f
        print "<ApplicationPack version=\"0.1\">"
        for k, v in self.app_pack.iteritems():
             print "    <%s>%s</%s>" % ( k, v, k )
        print "</ApplicationPack>"
        sys.stdout.flush()
        sys.stdout = backup
        f.flush()
        f.close()
        self.DetermineAvailablePacks()

''' Unit testing '''
def usage():
    print "Unit testing of pack.py"
    print "Usage: python pack.py <class> <file>"
    print "  <class> one of ApplicationPackParse, ApplicationPack"
    print "  <file> file to feed into the class as a unit"
    
if __name__ == "__main__":
    if len( sys.argv ) == 3:
        if sys.argv[1] == "ApplicationPack":
            print "Testing Class: ApplicationPack"
            print "  Parsing application pack from " + sys.argv[2]            
            apppack = ApplicationPack( INSTALL, sys.argv[2] )
        elif sys.argv[1] == "ApplicationPackParse":
            print "Testing Class: ApplicationPackParse"
            print "  Parsing application pack xml from " + sys.argv[2]
            pack = []
            handler = ApplicationPackParse( pack )
            parser = make_parser()
            parser.setContentHandler( handler )
            parser.parse( sys.argv[2] )
            printNestedStuff( pack )   
        else:
            print sys.argv[1] + " is not a valid class"
            usage()       
    else:
        usage()
