# A Windows Autorun file parser in Python for the WineDoors project.
# (c) Copyright Wine-Doors dev team 2008
#
# Probably doesn't work with mult-disc installers, but its a start.

import os, string, re
from utils import GetCD
from log import log

class Autorun:

    def __init__( self ):
        self.autorun_inf = None

    # Is there an autorun enable cd available?
    def detect_cd( self ):
        if not GetCD():
            log.Warn("No CD detected")
            return False
        for item in os.listdir( GetCD() ):
            if item.lower() == "autorun.inf":
                self.autorun_inf = GetCD() + "/" + item
                break
        if self.autorun_inf == None:
            log.Warn("No autorun.inf found on CD")
            return False
        return True

    # Looks for values in the inf
    # Usefull for looking up CD name/cheking disc number...
    def inf( self, value ):
        return_val = False
        if not self.autorun_inf and not self.detect_cd():
            log.Warn("Unable to retrive autorun information")
            return
        inf_value = re.compile( "(.*)=(.*)\\r$" )                     
        inf_file = open( self.autorun_inf, "r" )
        for line in inf_file:
            match = inf_value.search( line )
            if match and match.group( 1 ) == value:
                return_val = match.group( 2 )
        inf_file.close()
        return return_val

    # "Start" the disc
    def start(self, args=None):
        if not self.autorun_inf and not self.detect_cd():
            log.Warn("Unable to start disc")
            return
        if self.inf("open"):
            old_cwd = os.getcwd()
            os.chdir( GetCD() )
            os.system( "wine " + self.inf("open") )
            os.chdir( old_cwd )
        else:
            log.Warn("Autorun.inf does not specify an application to open")

autorun = Autorun()

# for debuggering:
if __name__ == "__main__":
    import sys

    # Detect CD
    if GetCD():
        print "CD detected: " + GetCD()
        # Detect autorun.inf
        autorun.detect_cd()
        if autorun.autorun_inf:
            print "Autorun File: " + autorun.autorun_inf
        else:
            print "No autorun file detected"
    else:
        print "No CD detected."
    sys.exit(0)
