import sys, os, time

from preferences import preferences

class Log:
    errors = []
    
    def __init__( self, logpath ):
        (path, logfile) = os.path.split( logpath )
        if not os.path.exists( logpath ):
            if not os.path.isdir( path ):
              os.makedirs( path )
            self.file = open( logpath, "w" )
            self.Log( "Created log file: " + logfile )
            self.Debug( "         in path: " + path )
        else: 
            self.file = open( logpath, "a" )
            self.Debug( "Opened log file: " + logfile )
            self.Debug( "        in path: " + path )
        self.Log( "Started logging session" )
        
    def __Format( self, message, type="" ):
        t = time.strftime( "%F %T" )
        return "%s %s: %s\n" % ( t, type, message )
    
    def Debug( self, message ):
        self.file.write( self.__Format( message, "Debug" ) )
        if preferences["debug"]:
            sys.stdout.write( "Debug: %s\n" % message )
            sys.stdout.flush()
        self.file.flush()
        
    
    def Log( self, message, flush = True ):
        self.file.write( self.__Format( message, "Info" ) )
        sys.stdout.write( "%s\n" % message )
        if flush: 
            self.file.flush()
            sys.stdout.flush()
    
    def Warn( self, message ):
        self.file.write( self.__Format( message, "Warning" ) );
        self.file.flush()
        sys.stderr.write( "Warning: %s\n" % message )
        self.file.flush()
    
    def Error( self, message ):
        self.file.write( self.__Format( message, "Error" ) );
        self.file.flush()
        sys.stderr.write( "Error: %s\n" % message )
        hit = False
        if len( self.errors ) > 0:
            for index, (e_message, count) in enumerate(self.errors):
                if message == e_message:
                    self.Debug("Dupe error message")
                    self.errors[index][1] += 1
                    hit = True
                    break
        else:
            self.errors += [[message, 1]]
            hit = True
        if not hit:
            self.errors += [[message, 1]]
    
    def __times( self, number ):
        return number == 1 and "1 time" or ( "%d times" % number )
    
    def GetErrors ( self ):
        return "".join( ["Reported %s: <i>%s</i>\n" % ( self.__times( count ), message ) for message, count in self.errors] )
    
    def ClearErrors ( self ):
        del self.errors[:]

# wineroot doesn't necessarily exist just yet so we can't put the log there
log = Log( os.path.expanduser( "~/.wine-doors/wine-doors.log" ) )

if __name__ == "__main__":
    log.Debug( "unittest" )
    log.Log( "unittest" )
    log.Warn( "unittest" )
    log.Error( "unittesting" )
    log.Error( "unittesting_twice" )
    log.Error( "unittesting_twice" )
    print log.getErrors()
    if log.getErrors() == "Reported 1 time <i>unittesting</i>\nReported 2 times <i>unittesting_twice</i>":
        print "OK"
    log.clearErrors()
    print log.getErrors()
    if log.getErrors() == "":
        print "OK"
