import sys, os
import gtk, gobject

from const import *

from preferences import preferences
from utils import Download
from utils import printNestedStuff
from utils import string2bool
from log import log
from copy import deepcopy
from wine import wine

from xml.sax import make_parser
from xml.sax.handler import ContentHandler


class PackListParse( ContentHandler ):
    def __init__( self, packs ):
        ContentHandler.__init__( self )
        self.in_app = False
        self.in_repo = False
        self.in_pack = False
        self.in_wdi = False
        self.v_data = []
        self.installersize = None
        self.installedsize = None
        self.wineversion = None
        self.winevariant = None
        self.appdb_rating = None
        self.appversion = None
        self.votes = None
        self.packs = packs 
        self.data = []
        self.requires = []
        self.majorversion = None
        self.pack = {}
        self.majorversions = [] 
        self.repo = []
        while len( self.repo ) < 2:
            self.repo.append( "" )
            
    def startElement( self, name, attrs ):
        if name == "Application":
            self.in_app = ( self.majorversion,
                            self.wineversion,
                            self.winevariant,
                            self.appdb_rating,
                            self.installersize,
                            self.installedsize,
                            self.votes,
                            self.appversion,
                            )
        elif self.in_app and name == "pack":
            self.in_pack = True
            self.majorversion = attrs.get( 'major_version' )
        elif self.in_pack and name == "wdi":
            self.in_wdi = True
            self.wineversion  = attrs.get( 'wineversion' )
            self.winevariant  = attrs.get( 'winevariant' )
            self.appdb_rating = attrs.get( 'appdb_rating' )
            self.votes        = attrs.get( 'votes' )
            self.appversion   = attrs.get( 'version' )
        elif name == "requires":
            self.requires.append( (attrs.get( 'name' ), attrs.get( 'version' )) )
        elif name == "repo":
            self.in_repo = True
            
    def endElement( self, name ):
        if name == "Application":
            self.in_app = False
            if "ms-license" not in self.pack:
                self.pack["ms-license"] = False
            self.pack['requires'] = self.requires
            self.requires = []
            self.pack["versions"] = self.majorversions
            self.pack["repo"] = self.repo
            self.packs.append( self.pack )
            self.data = []
            self.majorversion = None
            self.majorversions = []
            self.pack = {}
            return
        elif name == "repo":
            self.in_repo = False
            
        text = ''.join( self.data )
        
        if self.in_app and name == "pack":
            # Backward compatible for the time being
            if self.v_data == None:
                self.v_data = text
            # v_data is constructed from elements inside pack elements
            self.majorversions.append(self.v_data)
            self.v_data = []
            self.in_pack = False
            
        elif self.in_pack and name == "installersize":
            self.installersize = text
        elif self.in_pack and name == "installedsize":
            self.installedsize = text
        elif self.in_pack and name == "appdb":
            self.appdb = text
        elif self.in_pack and name == "wdi":
            self.v_data.append ( ( self.majorversion,
                                   self.wineversion, 
                                   self.winevariant, 
                                   self.appdb_rating, 
                                   self.installersize,
                                   self.installedsize,
                                   self.votes,
                                   self.appversion,
                                   text ) )
            self.majorversion = None
            self.wineversion = None
            self.winevariant = None
            self.appdb_rating = None
            self.installersize = None
            self.installedsize = None
            self.votes = None
            self.appversion = None
            self.in_wdi = False
        elif self.in_repo and name == "name":
            self.repo[0] = text
        elif self.in_repo and name == "uri":
            self.repo[1] = text
        elif self.in_app:
            self.pack[name] = text
        self.data = []
        
    def characters( self, string ):
        self.data.append( string.rstrip() )
        
    def error( self, exception ):
        log.Error( "%s\n" % exception )

class PackList: 
    def __init__( self, repo=None, fresh=False ):
        self.available = []
        self.all = []
        self.packs = []
        self.installed = []
        self.updates = []
        
        self.packlist_path = os.path.expanduser( "~/.wine-doors/packlists/" )
        if not os.path.isdir( self.packlist_path ):
            self.tmp = True
            self.fresh = fresh
        else:
            self.tmp = False
            self.fresh = fresh
        if not repo:
            pass  
        elif repo.startswith ( "http://" ):
            self.Update( repo )
        elif os.path.isfile( repo ):
            self.packs = self.Parse( repo )
            self.packrepo = None
            self.packfile = repo
        elif repo:
            self.Update( repo )
        
        wdroot = preferences["wineroot"] + "wine-doors/"
        if not os.path.isdir( wdroot ):
            log.Debug( "Creating path %s" % wdroot )
            os.makedirs( wdroot );

        self.GetInstalledPacks()
        self.DetermineAvailablePacks()

    def __contains__( self, key ):
        try:
            self.all.index( key )
            return True
        except IndexError:
            return False    

    def GetInstalledPacks ( self ):
        self.installed_list = preferences["wineroot"] + "wine-doors/installed.packlist.xml"
        if os.path.isfile( self.installed_list ):
            self.Parse( self.installed_list )
            self.installed = self.packs
        # Dont forget about non WDI's
        if string2bool( preferences['installedfromreg'] ):
            for installed in wine.installedFromReg():
                if not self.IsInstalled( installed['shortname'], None ):
                    self.installed.append( installed )
                    self.all.append( installed )

    def GetPack (self, pack_shortname ):
        log.Debug("Getting pack " + pack_shortname)
        for tmp_pack in self.all:
            if tmp_pack['shortname'] == pack_shortname:
                return tmp_pack
        log.Error("Failed to get pack " + pack_shortname)
        return None

    def GetRepo ( self, pack_shortname):
        log.Debug("Retrieving repo for pack " + pack_shortname )
        for tmp_pack in self.all:
            if tmp_pack['shortname'] == pack_shortname:
                return tmp_pack['repo']
        log.Error("Repo not found for pack " + pack_shortname )
        return None

    def GetRepoURI ( self, repo_name ):
        """
        Lookup the repository URI from the name
        """
        repo_uri = ""
        for item in preferences.repositories:
            if ''.join( item[0:1] ) == repo_name:
                repo_uri = ''.join( item[1:] )
        return repo_uri
        
    def GetRepoName ( self, repo_uri ):
        """
        Look up the repository name from the URI
        """
        repo_name = None
        for item in preferences.repositories:
            if ''.join( item[1:] ) == repo_uri:
                repo_name = ''.join( item[0:1] )
        return repo_name

    def CheckRequires( self, pack_shortname, deps=[] ):
        for tmp_pack in self.all:
            if tmp_pack['shortname'] == pack_shortname:
                for dep in tmp_pack['requires']:
                    deps.append( dep )
        return deps

    def Parse( self, xmlfile ):
        """
        Invoke a SAX parser for PackList XML
        """
        self.packs = []
        if not os.path.isfile ( xmlfile ):
            log.Error( "No such file : " + xmlfile )
        handler = PackListParse( self.packs )
        parser = make_parser()
        parser.setContentHandler( handler )
        parser.parse( xmlfile )
    
    def Update( self, repo ):
        """
        Retrieve an XML list of application packs and their URLs
        """
        self.packs = []
        repo_uri = self.GetRepoURI ( repo )
        supplied_repo = repo
        if not repo_uri:
            log.Debug( repo + " is not a valid repository" )
            log.Debug( "  attempting a reverse lookup" )
            repo = self.GetRepoName( repo )
            repo_uri = self.GetRepoURI( repo )
        if not repo:
            log.Error( "Repo update failed for " + supplied_repo )
            
        elif not repo.startswith ( "http://" ):
            repo_uri += "/packlist.xml.gz"
        try:
            log.Debug( "Checking for available applications in " + repo )
        except:
            log.Error( "Repo update failed for " + supplied_repo )
            return
        if self.tmp:
            local_file = "/tmp/%s.xml" % repo
        else:
            if not os.path.isdir( self.packlist_path ):
                log.Debug( "Creating path " + self.packlist_path )
                os.makedirs(self.packlist_path);
            local_file = self.packlist_path
            local_file += repo + ".xml"
        Download( repo_uri, local_file + ".gz", not self.fresh )
        self.packrepo = repo
        if os.path.isfile( local_file):
            log.Debug( "Parsing packlist " + local_file )
            self.Parse( local_file )
        else:
            log.Error( "Could not find packlist " + local_file )
            return

        self.all += self.packs
        
        if self.tmp:
            os.remove( local_file )
            
    def UpdateAll ( self ):
        self.available = []
        self.all = []
        self.packs = []
        self.installed = []
        
        for item in preferences.repositories:
            self.Update( ''.join( item[0:1] ) )

        self.packs = []
        
        if os.path.isfile( self.installed_list ):
            self.Parse( self.installed_list )
            self.installed = self.packs

        # Dont forget about non WDI's
        if string2bool( preferences['installedfromreg'] ):
            for installed in wine.installedFromReg():
                if not self.IsInstalled( installed['shortname'], None ):
                    self.installed.append( installed )
                self.all.append( installed )
        self.DetermineAvailablePacks()
        self.TidyPacks()
        #printNestedStuff( self.all )
    
    def TidyPacks(self):
        new_all = []
        for pack in self.all:
            if "".join(pack['shortname']) != "" and \
            wine.WineVersion().split(".") >= pack['versions'][0][0][1].split("."):
                    new_all.append(pack)
        self.all = new_all
        new_available = []
        for pack in self.available:
            if "".join(pack['shortname']) != "" and \
            wine.WineVersion().split(".") >= pack['versions'][0][0][1].split("."):
                    new_all.append(pack)
                    new_available.append(pack)
        self.available = new_available
        new_installed = []
        for pack in self.installed:
            if "".join(pack['shortname']) != "":
                new_installed.append(pack)
        self.installed = new_installed
        
    def AddInstalled( self, pack, version, revision=None ):
        if self.IsInstalled( pack, version, None ):
            self.DelInstalled( pack, version )
        done=False
        for pack_a in self.all:
            if pack_a['shortname'] == pack:
                for pack_version in pack_a['versions']:
                    [( majorversion, 
                       wineversion, 
                       winevariant, 
                       appdb_rating, 
                       installersize, 
                       installedsize, 
                       votes,
                       minor_version,
                       app_pack )] = pack_version
                    if version in (majorversion, minor_version):
                        pack_ins = deepcopy(pack_a)
                        pack_ins['versions'] = []
                        pack_ins['versions'].append(pack_version)
                        log.Debug("Adding installed application: " + pack + "-" + version )
                        self.installed.append( pack_ins )
                        done=True
                        break
            if done:
                break
        if done:
            self.Save(self.installed_list)
        else:
            log.Debug("Not saving, nothing was updated")
        
    def DelInstalled( self, pack, version ):
        for pack_b in self.installed:
            if pack_b['shortname'] == pack:
                for pack_version in pack_b['versions']:
                    [( majorversion, 
                       wineversion, 
                       winevariant, 
                       appdb_rating, 
                       installersize, 
                       installedsize, 
                       votes,
                       minor_version,
                       app_pack )] = pack_version
                    if version in (majorversion, minor_version):
                        self.installed.remove(pack_b)
        self.Save(self.installed_list)
    
    def Save( self, tofile = "installed.packlist.xml" ):
        """ 
        Save to installed.packlist.xml 
        """
        log.Debug("Saving installed list: " + tofile )
        f = open( tofile, "w" )
        backup = sys.stdout
        sys.stdout = f
        print "<PackList version=\"0.1\">"
        for pack in self.installed:

            # Dont save non-wdi's (They may change between wine-doors launches)
            if pack.has_key("UninstallString"):
                continue

            print "  <Application>"
            for k, v in pack.iteritems():
                if k == "versions":
                    for pack_version in pack['versions']:
                        [( majorversion, wineversion, winevariant, appdb_rating, installersize, installedsize, votes, version, app_pack )] = pack_version
                        
                        print "    <pack major_version=\"%s\">" % majorversion
                        print "      <installersize>%s</installersize>" % installersize
                        print "      <installedsize>%s</installedsize>" % installedsize
                        print "      <wdi wineversion=\"%s\" winevariant=\"%s\" appdb_rating=\"%s\" votes=\"%s\" version=\"%s\">%s</wdi>" % ( wineversion,  winevariant, appdb_rating, votes, version, app_pack )
                        try:
                            print "      <appdb>%s</appdb>" %  appdb
                        except:
                            pass
                        print "    </pack>"
                elif k == "repo":
                    print "    <%s>%s</%s>" % ( k, "".join(v[0:1]), k )
                elif k == "requires":
                    for dep in pack['requires']:
                        try:
                            [(dep_name, dep_version)] = dep
                            print "    <requires name=\"%s\" version=\"%s\" />" % (dep_name, dep_version)
                        except:
                            pass
                else:
                    print "    <%s>%s</%s>" % ( k, v, k )
            print "  </Application>"
        print "</PackList>"
        sys.stdout.flush()
        sys.stdout = backup
        f.flush()
        f.close()
        self.DetermineAvailablePacks()
    
    def SetRating( self, pack_source, pack_shortname, uversion, urating ):
        if pack_source == SHOW_ALL:
            packs = self.all
        elif pack_source == SHOW_AVAILABLE:
            packs = self.all
        elif pack_source == SHOW_UPDATES:
            packs = self.all
        elif pack_source == SHOW_INSTALLED:
            packs = self.installed
        elif pack_source == SHOW_QUEUE:
            packs = self.all
        else:
            packs = packlist.all
            
        packlist = []
        pack_versions = []
        for pack in packs:
            if pack['shortname'] != pack_shortname:
                packlist.append(pack)
                continue
            for pack_version in pack['versions']:
                [( majorversion, wineversion, winevariant, appdb_rating, installersize, installedsize, votes, version, app_pack )] = pack_version
                if majorversion != uversion:
                    pack_versions.append(pack_version) 
                    continue
                vdata = [ ( majorversion, wineversion, winevariant, appdb_rating, installersize, installedsize, urating, version, app_pack ) ]
                pack_versions.append ( vdata )
                pack['versions'] = pack_versions
                packlist.append(pack)
                # Submit this data back to wine-doors.org???
        
        self.DetermineAvailablePacks()
        
        if pack_source == SHOW_ALL:
            self.all = packlist
        elif pack_source == SHOW_AVAILABLE:
            self.all = packlist
        elif pack_source == SHOW_UPDATES:
            self.all = packlist
        elif pack_source == SHOW_INSTALLED:
            self.installed = packlist
        elif pack_source == SHOW_QUEUE:
            self.all = packlist
        else:
            self.all = packlist
        
        if pack_source != SHOW_AVAILABLE:
            self.DetermineAvailablePacks()


    def CompareVersions(self, va, vb):
        ''' Returns latest version'''
        versions = [va,vb]
        ntotal=0
        for version in versions:
                version = map(int, version.split('.'))
                ototal=0
                curspot=10000000
                for y in version:
                        ototal+=y*curspot
                        curspot/=100
                if ototal>=ntotal:
                        ntotal=ototal
                        newest=version
        return '.'.join(map(str, newest))
    
    def DetermineAvailablePacks( self ):
        """
        Xor the packs all and installed to produce available
        """
        log.Debug( "Xoring all/installed to create available" )
        self.available = []
        for pack in self.all:
            if pack['shortname'] == "":
                continue
            newpack = deepcopy(pack)
            available = False
            versions = pack['versions']
            newpack['versions'] = []
            for pack_version in versions:
                [( av_version, 
                   wineversion, 
                   winevariant, 
                   appdb_rating, 
                   installersize, 
                   installedsize, 
                   votes,
                   av_revision,
                   app_pack )] = pack_version
                
                if not self.IsInstalled(pack['shortname'], av_version) and not self.InAvailable(pack['shortname'] ):
                    available = True
                    newpack['versions'].append(pack_version)
    
            if available:
                self.available.append( newpack )
    
    def IsInstalled (self, pack_shortname, version, revision=None):
        for tmp_pack in self.installed:
            if tmp_pack['shortname'] == pack_shortname:
                if not version:
                    return True
                for pack_version in tmp_pack['versions']:
                    [( inst_version, 
                       wineversion, 
                       winevariant, 
                       appdb_rating, 
                       installersize, 
                       installedsize, 
                       votes,
                       inst_revision,
                       app_pack )] = pack_version
                    if version in (inst_version, inst_revision):
                        return True
        return False

    def InAvailable (self, pack_shortname, version=None ):
        for tmp_pack in self.available:
            if tmp_pack['shortname'] == pack_shortname:
                if not version:
                    return True
                for pack_version in tmp_pack['versions']:
                    [( av_version, 
                       wineversion, 
                       winevariant, 
                       appdb_rating, 
                       installersize, 
                       installedsize, 
                       votes,
                       av_revision,
                       app_pack )] = pack_version
                    if version in (av_version, av_revision):
                        return True
        return False

    def InAll (self, pack_shortname, version=None ):
        for tmp_pack in self.all:
            if tmp_pack['shortname'] == pack_shortname:
                if not version:
                    return True
                for pack_version in tmp_pack['versions']:
                    [( av_version, 
                       wineversion, 
                       winevariant, 
                       appdb_rating, 
                       installersize, 
                       installedsize, 
                       votes,
                       av_revision,
                       app_pack )] = pack_version
                    if version in (av_version, av_revision):
                        return True
        return False
            
    def InPacks (self, pack_shortname ):
        for tmp_pack in self.packs:
            if tmp_pack['shortname'] == pack_shortname:
                return True
        return False
     
#-------- FFR
    def MergePacklists( self, packlist_a, packlist_b):
        """
        Merge two packlist structures, packlist_a overrides packlist_b
        """
        # return packs
        pass

    def SubmitVote( self, pack, version, vote ):
        pass

    def IsUpdate( self, pack, version ):
        return False

    def RequiresLicense( self, pack, version ):
        return True

    def HaveLicense( self, pack, version ):
        return True

    def AddLicense( self, pack, version, license_key ):
        pass

    def RequestLicense( self, pack, version ):
        """
        Request a license from a repository owner
        """
        
        pass
    
''' Unit testing '''   
def usage():
    print "Unit testing of pack.py"
    print "Usage: python pack.py <class> <file>"
    print "  <class> one of PackListParse, PackList"
    print "  <file> file to feed into the class as a unit"    
    
if __name__ == "__main__":
    if len( sys.argv ) == 3:
        if sys.argv[1] == "PackList":
            print "Testing Class: PackList"
            print "  Parsing packlist from " + sys.argv[2]
            packlist.Update ( sys.argv[2] )
            print 
            printNestedStuff( packlist.packs )
        elif sys.argv[1] == "PackListParse":
            print "Testing Class: PackListParse"
            print "  Parsing packlist xml from " + sys.argv[2]
            xpacks = []
            xhandler = PackListParse( xpacks )
            xparser = make_parser()
            xparser.setContentHandler( xhandler )
            xparser.parse( sys.argv[2] )
            printNestedStuff( xpacks )
        else:
            print sys.argv[1] + " is not a valid class"
            usage()       
    else:
        usage()
else:
    packlist = PackList()  
