
import os, re, time

from const import *

from application import ApplicationPack
from preferences import preferences
from packlist import packlist
from wine import wine
from log import log

PACK_NAME, PACK_SHORTNAME = range ( 2 )

class Queue:
    """
    I think wine-doors needs to be centralised by the queue, it seem to me to be the 
    interchange point between the user interface and the packages... some refactoring 
    needs to be worked on.
    """
    def __init__( self ):
        self.gui = True
        self.queue = []
        self.FirstRun = False

    def Add( self, action, pack=None, version=None, repo=None ):
        if not packlist.InAll( pack, version ):
            log.Error( "Attempted to add pack \"" + str(pack) + \
                       "\" to queue which is not in packlist.all" )
            return
        if not self.IsOnQueue( pack, version, action ):
            if self.IsOnQueue( pack, version ):
                self.Delete( pack, version )
            log.Debug("Adding to queue %s version %s" % (pack, version))
            self.queue.append ( ( action, pack, version, repo ) )
            

    def IsOnQueue(self, pack, version, foraction=None ):
        for item in self.queue:
            action, this_pack, this_version, repo = item
            if pack == this_pack and version == this_version:
                if foraction:
                    if action == foraction:
                        return True
                else:
                    return True  
        return False
    
    def Delete( self, pack, version ):
        temp_q = []
        log.Debug("Removed from queue %s version %s" % (pack, version))
        for item in self.queue:
            action, this_pack, this_version, repo = item
            if pack != this_pack and version != this_version:
                temp_q.append ( (action, this_pack, this_version, repo) )
        self.queue = temp_q
    
    def SetProgress( self, index = 0, widget=None ):
        """
        Set the current progress of the queue depending on how we're being 
        executed. However this does not function as expected for many reasons
        mostly CLI is broken
        """
        try:
            if self.gui:
                ui.benign()
        except:
            import ui

        if widget:
            self.widget = widget
        total = len( self.queue )
        if index >= total: 
            return False
            
        if index == -1:
            index = 1
            total = 1
        
        action, pack, version, repo = self.queue[index]
        
        if action == DRIVE:
            name = "Creating wine drive"
        elif action == UNINSTALL:
            tmp_pack = packlist.GetPack( pack )
            data = tmp_pack['name']
            name = "Uninstalling Application Pack %s" % data
        else:
            tmp_pack = packlist.GetPack( pack )
            data = tmp_pack['name']
                    
            if "default_version" in tmp_pack:
                version = tmp_pack['default_version']
                name = "Installing Application Pack %s %s" % ( data, version )
            else:
                version = ALL
                name = "Installing Application Pack %s all versions" %  data
        
        percentage = float( index ) / float ( total )
        if self.gui:
            ui.SetProgress( percentage, WIDGET, name, self.widget)
        return False
        
    def Execute( self , FirstRun=False ):
        """
        Queue processing, this is already pretty good, but I'd like
        to remove the first run thing and have a different processing method
        for initialising a bottle. As this is essentially what we're doing
        with 0.2 we shouldn't have to install anything more than once
        """
        self.FirstRun = FirstRun
        try:
            if self.gui:
                ui.benign()
        except:
            import ui, gtk, gobject

        if FirstRun and self.gui:
            self.widget = ui.window['progressbar']
        elif self.gui:
            self.widget = ui.winedoors.window['progressbar']

        self.ResolveDependencies()

        if FirstRun:
            self.SetProgress( 0 )
            self.widget.queue_draw()

        print "Executing queue"
        for item in self.queue:
            action, pack, version, repo = item   

            if action == UNINSTALL:
                log_text  = "Uninstalling Application: "
                log_text += pack
                status_text = "Uninstalling..."
            else:
                log_text  = "Installing Application: "
                log_text += pack
                if not version:
                    version = None
                else:
                    log_text += " version " + version
                status_text = "Installing..."

            self.SetProgress( self.queue.index( item ) )

            if not self.FirstRun and self.gui:
                try:
                    if self.gui:
                        ui.benign()
                except:
                    import ui
                ui.winedoors.tree.anim = True

                ui.winedoors.tree.set_cell_status((0,), status_text )
                #ui.winedoors.tree.throb_cell( (0,) )
                #gobject.timeout_add(200, ui.winedoors.tree.throb_cell, (0,) )
            log.Log( log_text )
            
            if action == INSTALL:
                app_pack = ApplicationPack( INSTALL, pack, version, repo )
                if app_pack.status:
                    log.Debug( "Error code: %d " % app_pack.status)
                    log.Error( "Installation failed: %s returned %d" % ( log_text, app_pack.status ) )
            elif action == UPDATE:
                app_pack = ApplicationPack( UPDATE, pack, version, repo )
                if app_pack.status:
                    log.Debug( "Error code: %d " % app_pack.status)
                    log.Error( "Update failed: " + log_text )
            elif action == UNINSTALL:
                app_pack = ApplicationPack( UNINSTALL, pack, version, repo )
                # Non WDI's don't have a status attribute
                if hasattr(app_pack, "status") and app_pack.status:
                    log.Debug( "Error code: %d " % app_pack.status)
                    log.Error( "Uninstall failed: " + log_text )

            if not self.FirstRun and self.gui and action != DRIVE:
                ui.winedoors.tree.anim = False
                for i in xrange(10):                
                    ui.winedoors.tree.fade_cell((0,), "Complete...", "installed", 10)
                    time.sleep(0.1)

            self.SetProgress(self.queue.index( item ) + 1 )

            yield True

        if self.gui:
            ui.Complete( self.widget )

            if FirstRun:
                ui.window['wn_firstrun'].hide()
                gtk.main_quit()
                
        wine.Reboot()
        self.queue = []
        yield False

    def ResolveDependencies( self ):
        """
        Dependency Resolution
        This is the simplest and yet apparently most effective route. 
        By reversing the queue every time I Add an item to the queue list
        it effectively prepends the item to the list ensuring it will be 
        installed first. Then a simple postumous reverse and voila!
        """
        try:
            if self.gui:
                ui.benign()
        except:
            import ui, gtk

        if not self.FirstRun and self.gui:
            ui.winedoors.window['progressbar'].set_text("Resolving dependencies")
            ui.winedoors.window['progressbar'].queue_draw()

        self.queue.reverse()
        for item in self.queue:
            action, pack, version, repo = item
            if action == UNINSTALL or action == DEPS:
                continue

            log.Log("Resolving dependencies for %s " % pack )
            deps = packlist.CheckRequires(pack, [])
            
            for dep in deps:
                if not self.FirstRun and self.gui:
                    ui.winedoors.window['progressbar'].pulse()
                    ui.winedoors.window['progressbar'].queue_draw()
                (dpack, dversion) = dep
                # Add each dependency to the start of the queue
                if not packlist.IsInstalled(dpack, None) and not self.IsOnQueue(dpack, dversion, INSTALL):
                    self.Add(INSTALL, dpack, dversion )
                    log.Log("Added dependency %s version %s for %s version %s" % (dpack, dversion, pack, version))
        self.queue.reverse()

        if not self.FirstRun and self.gui:
            ui.winedoors.tree.widget.set_model( ui.winedoors.tree.CreateModel(self.queue) )

    def AddBaseLibraries( self ):
        """ Pulls all the firstrun files to install from System Base, how lame!
        I hate this with a passion, so we need to create an initialise bottle function
        we can inherit from this bottle for all subsequent bottles.
        """
        pass
        repo = "System Base"
        packlist.Update( repo )
        for pack in packlist.packs:
            licenserqd = pack["ms-license"]
            if not licenserqd or preferences['licensed']:
                self.Add( INSTALL, pack["shortname"], None, repo )
            
queue = Queue()
