#!/usr/bin/env python
#
# @author: Karl Lattimer <karl@qdh.org.uk>
# @package: wine-doors
# @license: GNU GPLv2
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
import os, sys
from wine import wine
from queue import queue
from application import ApplicationPack
from preferences import preferences
from const import *
from utils import ClearCache

# Change process name
try:
    # Linux
    import dl
    libc = dl.open('/lib/libc.so.6')
    libc.call('prctl', 15, 'wine-doors', 0, 0, 0)
except:
    try:
        import ctypes
        libc = ctypes.CDLL('libc.so.6')
        libc.prctl(15, 'wine-doors', 0, 0, 0)
    except:
        try:
            # BSD
            import ctypes
            libc = ctypes.CDLL('libc.so')
            libc.setproctitle('wine-doors')
        except:
            # Fail
            from log import log
            log.Warn("""Cannot change process name.  Please file a bug report at 
         http://www.wine-doors.org/trac/newticket. 
         Include OS name and version.
         This is a known issue on Solaris.""")

def usage( name ):
    print "Usage: " + name + " <command> <argument>"
    print "\ncommands:"
    print "    --usage,--help     present this help information"
    print "    --list             show installed applications"
    print "    --install          install an application (<application name>)"
    print "    --uninstall        uninstall an application (<application name> [version])"
    print "    --install-all      install all applications (<repository url>)"
    print "    --add-repository   add a repository (<repository url>\n"
    print "    --firstrun         run the firstrun process\n"
    print "    --createbottle     Create a new wine-bottle <wineprefix>\n"
    print "    --usebottle        Use a specified bottle <wineprefix>\n"
    print "    --nogui            Don't import gtk or ui\n"
    print "    --clear-cache      Clear the package and icon cache"
    print "    --name             Set name in wine-doors preferences (<name>)"
    print "    --company          Set company in wine-doors preferences (<company name>)"

if __name__ == "__main__":
    try:
        dofirstrun = False
        wine.firstrun = False
        gui = True
    
        for arg in xrange(len(sys.argv)):
            if ( ( sys.argv[arg] == '--help' ) or ( sys.argv[arg] == '--usage' ) ):
                usage( sys.argv[0] )
                gui = False
            elif ( sys.argv[arg] == '--list' ):
                gui = False
                print 'Installed:'
                from packlist import packlist
                packlist.GetInstalledPacks()
                for installed in packlist.installed:
                    print "  " + installed['shortname'] + " " + installed['default_version']
            elif ( sys.argv[arg] == '--install' ):
                gui = False
                ApplicationPack( INSTALL, sys.argv[2] )
            elif ( sys.argv[arg] == '--uninstall' ):
                gui = False
                ApplicationPack( UNINSTALL, sys.argv[arg+1], sys.argv[arg+2])
            elif ( sys.argv[arg] == '--install-all' ):
                print 'FIXME: install-all'
                gui = False
            elif ( sys.argv[arg] == '--add-repository' ):
                gui = False
                print 'FIXME: add-repository'
            elif ( sys.argv[arg] == '--firstrun' ):
                dofirstrun = True
            elif ( sys.argv[arg] == '--createbottle' ):
                preferences['wineroot'] = os.path.abspath(sys.argv[arg+1])
                if not preferences['wineroot'].endswith("/"):
                    preferences['wineroot'] += "/"
                preferences.Save()
                dofirstrun = True
            elif ( sys.argv[arg] == '--usebottle' ):
                preferences['wineroot'] = os.path.abspath(sys.argv[arg+1])
                if not preferences['wineroot'].endswith("/"):
                    preferences['wineroot'] += "/"
                preferences.Load()
            elif ( sys.argv[arg] == '--nogui' ):
                gui = False
            elif ( sys.argv[arg] == '--clear-cache' ):
                ClearCache()
            elif ( sys.argv[arg] == '--name ' ):
                preferences['name'] = sys.argv[arg+1]
                preferences.Save()
            elif ( sys.argv[arg] == '--company ' ):
                preferences['company'] = sys.argv[arg+1]
                preferences.Save()
            elif sys.argv[arg].startswith("--"):
                print sys.argv[0] + ": Error unknown argument"
                usage( sys.argv[0] )
                gui = False
                dofirstrun = False
                break

        if gui:
            import gtk, ui
            os.chdir( os.path.dirname( __file__ ) or "." )
            if not wine.CheckDrive() or dofirstrun: 
                wine.firstrun = True
                ui.window = ui.Window( "wn_firstrun", ui.GH_FirstRun )
                gtk.main()
                wine.firstrun = False
            ui.winedoors = ui.WineDoorsGUI()
            gtk.main()
        elif dofirstrun and not gui:
            queue.gui = False
            wine.gui = False
            wine.firstrun = True
            # Can i now run queue.AddBaseLibs... etc...
            wine.CreateDrive()
        else:
            queue.gui = False
            wine.gui = False
    except KeyboardInterrupt:
        print "c^ caught.  Quitting."
        sys.exit(0)
        #TODO: Check whats happening before quiting.
